
# q-expansion of newform 9248.2.a.l, downloaded from the LMFDB on 06 February 2025.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 9248
weight = 2
poly_data = [-2, 0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0, 0], [0, 1], [0, -2], [0, 0], [0, 3], [2, 0], [0, 0], [-4, 0], [0, 2], [0, -2], [0, 0], [0, 0], [0, -7], [2, 0], [12, 0], [6, 0], [-10, 0], [0, 0], [14, 0], [0, 10], [0, -9], [0, 2], [-6, 0], [-8, 0], [0, 3], [-18, 0], [4, 0], [0, -1], [0, 2], [0, -3], [-12, 0], [0, -11], [-6, 0], [0, 11], [-2, 0], [-8, 0], [-18, 0], [0, -1], [0, 12], [0, -18], [-12, 0], [0, 12], [-24, 0], [0, 7], [0, 8], [0, -6], [0, -13], [-20, 0], [0, 19], [-2, 0], [0, 1], [-16, 0], [0, 9], [18, 0], [12, 0], [28, 0], [0, -18], [-16, 0], [0, 6], [-22, 0], [0, -17], [22, 0], [-4, 0], [0, -24], [0, -13], [0, 8], [-10, 0], [0, -11], [0, 13], [-22, 0], [-20, 0], [-24, 0], [0, 10], [26, 0], [0, 3], [-16, 0], [-18, 0], [0, -20], [0, 11], [-32, 0], [0, -17], [6, 0], [0, 22], [-2, 0], [0, -14], [10, 0], [0, 3], [6, 0], [-14, 0], [-32, 0], [12, 0], [0, -10], [0, -30], [20, 0], [0, -17], [0, 14], [6, 0], [0, -27], [-18, 0], [0, 16], [0, 19], [34, 0], [-10, 0], [-20, 0], [0, 9], [-18, 0], [4, 0], [24, 0], [-40, 0], [0, 25], [0, -12], [-38, 0], [0, 23], [0, -5], [8, 0], [0, 13], [0, -13], [-16, 0], [0, 6], [12, 0], [-46, 0], [0, 9], [0, 4], [0, -21], [0, 3], [50, 0], [0, -6], [0, 10], [20, 0], [2, 0], [4, 0], [0, 24], [0, 30], [26, 0], [-30, 0], [34, 0], [34, 0], [0, -21], [42, 0], [0, -5], [0, -11], [0, 16], [0, -34], [0, 25], [50, 0], [0, -4], [0, 6], [0, -3], [6, 0], [24, 0], [0, 22], [0, 7], [26, 0], [0, -18], [0, 3], [0, 4], [-4, 0], [0, 35], [-10, 0], [0, 0], [0, 27], [-36, 0], [-16, 0], [-30, 0], [-44, 0], [0, 2], [0, 22], [0, 24], [0, 3], [0, 4], [20, 0], [10, 0], [0, 8], [-8, 0], [32, 0], [0, 27], [0, -3], [0, 4], [-12, 0], [-10, 0], [-44, 0], [0, 11], [0, 16], [48, 0], [-4, 0], [-18, 0], [0, -20], [-36, 0], [0, -33], [0, 20], [0, 3], [0, 23], [60, 0], [42, 0], [0, 11], [0, 7], [0, -27], [0, -20], [0, 1], [36, 0], [0, -28], [0, -14], [-6, 0], [28, 0], [28, 0], [50, 0], [4, 0], [36, 0], [0, 27], [-28, 0], [0, -23], [18, 0], [0, 16], [28, 0], [0, -16], [0, 13], [28, 0], [30, 0], [0, -28], [30, 0], [-26, 0], [0, -28], [-70, 0], [0, -4], [-30, 0], [-38, 0], [0, -19], [0, 2], [-48, 0], [0, 13], [-30, 0], [0, 23], [-44, 0], [-18, 0], [66, 0], [40, 0], [0, 5], [0, 20], [0, -47], [40, 0], [0, -7], [28, 0], [-4, 0], [10, 0], [0, -5], [0, -20], [0, -12], [0, -13], [-10, 0], [76, 0], [-30, 0], [0, 13], [60, 0], [0, -37], [38, 0], [54, 0], [0, 52], [0, 39], [0, 32], [64, 0], [0, -24], [-68, 0], [0, 4], [0, 38], [0, -53], [-70, 0], [54, 0], [-52, 0], [0, -19], [-30, 0], [0, -34], [-20, 0], [80, 0], [52, 0], [2, 0], [44, 0], [6, 0], [-46, 0], [-64, 0], [-14, 0], [52, 0], [0, 32], [0, -40], [-18, 0], [0, 21], [44, 0], [0, -5], [0, 28], [-20, 0], [-50, 0], [0, -26], [0, -41], [-20, 0], [0, -31], [0, 24], [0, 50], [4, 0], [-26, 0], [0, 13], [22, 0], [76, 0], [-62, 0], [0, 40], [0, 11], [0, -7], [0, -3], [12, 0], [0, 0], [-12, 0], [38, 0], [0, -14], [0, 38], [0, -27], [4, 0], [-12, 0], [-74, 0], [-68, 0], [0, -56], [0, -35], [20, 0], [0, -27], [0, 9], [-2, 0], [-40, 0], [0, -39], [-28, 0], [0, -17], [0, -45], [0, 34], [0, -32], [0, 6], [0, -24], [0, 50], [36, 0], [0, 13], [0, -11], [50, 0], [0, 41], [0, 27], [52, 0], [-42, 0], [68, 0], [0, 60], [4, 0], [-14, 0], [0, -25], [0, 12], [-62, 0], [0, -16], [0, -40], [6, 0], [0, 61], [-50, 0], [0, 52], [14, 0], [6, 0], [12, 0], [0, -39], [0, -9], [-48, 0], [0, 28], [0, -47], [4, 0], [0, -43], [-12, 0], [26, 0], [0, 38], [-32, 0], [0, -17], [30, 0], [0, -31], [0, -6], [34, 0], [80, 0], [0, 34], [0, 21], [0, -68], [-60, 0], [14, 0], [-58, 0], [0, 14], [-74, 0], [0, 58], [0, 55], [0, -57], [0, 60], [-100, 0], [-58, 0], [0, -5], [-4, 0], [0, 7], [0, -32], [68, 0], [-44, 0], [-32, 0], [0, -15], [80, 0], [22, 0], [0, 69], [42, 0], [0, 28], [6, 0], [-36, 0], [0, 65], [-30, 0], [0, 36], [-18, 0], [54, 0], [50, 0], [0, 47], [0, -3], [78, 0], [60, 0], [0, 31], [76, 0], [0, -56], [0, -22], [0, 48], [0, 25], [48, 0], [-18, 0], [0, -30], [0, 2], [-60, 0], [0, -21], [-70, 0], [0, -11], [0, 1], [-46, 0], [0, 32]]
